@@ -22,6 +22,20 @@
 #include "hlrmisc.h"
 #include "avlTree.h"
 
+/* Union for C99-compliant conversion between void* and function pointers.
+   This avoids pedantic warnings about void* to function pointer casts. */
+typedef union {
+  void *ptr;
+  void (*func0)(void*);
+  void (*func1)(void*,void*);
+  void (*func2)(void*,void*,void*);
+  void (*func3)(void*,void*,void*,void*);
+  void (*func4)(void*,void*,void*,void*,void*);
+  void (*func5)(void*,void*,void*,void*,void*,void*);
+  void (*func6)(void*,void*,void*,void*,void*,void*,void*);
+  void (*funcva)(void*,int,va_list);
+} AvlFuncPtr;
+
 static void avl_nodeCreate (AvlNode node) {
   node->leftNode = NULL;
   node->rightNode = NULL;
@@ -699,8 +713,8 @@ int avl_treeHeight (AvlTree this1) {
   return avl_nodeHeight (this1->rootTree);
 }
 
-static void avl_nodeCallFunction (AvlNode node,void *applyFunc(),
-                                  int nargs,va_list args){
+static void avl_nodeCallFunction (AvlNode node,void *applyFunc,int nargs,
+                                  va_list args){
   /*
     Only for internal use. avl_nodeApplyFunc() should be called to
     access this function.
@@ -708,21 +722,23 @@ static void avl_nodeCallFunction (AvlNode node,void *applyFunc(),
     should be able to parse  variable argument list in the same way
     as shown below.
   */
+  AvlFuncPtr fp;
+  fp.ptr = applyFunc;
   switch (nargs) {
   case 0:
-    applyFunc (node->value);
+    fp.func0(node->value);
     break;
   case 1:
     {
       void *arg1 = va_arg (args,void *);
-      applyFunc (node->value,arg1);
+      fp.func1(node->value,arg1);
       break;
     }
   case 2:
     {
       void *arg1 = va_arg (args,void *);
       void *arg2 = va_arg (args,void *);
-      applyFunc (node->value,arg1,arg2);
+      fp.func2(node->value,arg1,arg2);
       break;
     }
   case 3:
@@ -730,7 +746,7 @@ static void avl_nodeCallFunction (AvlNode node,void *applyFunc(),
       void *arg1 = va_arg (args,void *);
       void *arg2 = va_arg (args,void *);
       void *arg3 = va_arg (args,void *);
-      applyFunc (node->value,arg1,arg2,arg3);
+      fp.func3(node->value,arg1,arg2,arg3);
       break;
     }
   case 4:
@@ -739,7 +755,7 @@ static void avl_nodeCallFunction (AvlNode node,void *applyFunc(),
       void *arg2 = va_arg (args,void *);
       void *arg3 = va_arg (args,void *);
       void *arg4 = va_arg (args,void *);
-      applyFunc (node->value,arg1,arg2,arg3,arg4);
+      fp.func4(node->value,arg1,arg2,arg3,arg4);
       break;
     }
   case 5:
@@ -749,7 +765,7 @@ static void avl_nodeCallFunction (AvlNode node,void *applyFunc(),
       void *arg3 = va_arg (args,void *);
       void *arg4 = va_arg (args,void *);
       void *arg5 = va_arg (args,void *);
-      applyFunc (node->value,arg1,arg2,arg3,arg4,arg5);
+      fp.func5(node->value,arg1,arg2,arg3,arg4,arg5);
       break;
     }
   case 6:
@@ -760,28 +776,28 @@ static void avl_nodeCallFunction (AvlNode node,void *applyFunc(),
       void *arg4 = va_arg (args,void *);
       void *arg5 = va_arg (args,void *);
       void *arg6 = va_arg (args,void *);
-      applyFunc (node->value,arg1,arg2,arg3,arg4,arg5,arg6);
+      fp.func6(node->value,arg1,arg2,arg3,arg4,arg5,arg6);
       break;
     }
   default:
-    applyFunc (node->value,nargs,args);
+    fp.funcva(node->value,nargs,args);
   }
 }
 
-static void avl_nodeApplyFunc (AvlNode rootNode,void (*applyFunc)(),
-                               int nargs,va_list args) {
+static void avl_nodeApplyFunc (AvlNode rootNode,void *applyFunc,int nargs,
+                               va_list args) {
   if (rootNode == NULL)
     return;
   avl_nodeApplyFunc (rootNode->leftNode,applyFunc,nargs,args);
   va_list tmpArgs;
-  *tmpArgs = *args;
-  avl_nodeCallFunction (rootNode,(void*(*)())applyFunc,nargs,args);
-  *args = *tmpArgs;
+  va_copy(tmpArgs,args);
+  avl_nodeCallFunction (rootNode,applyFunc,nargs,tmpArgs);
+  va_end(tmpArgs);
   avl_nodeApplyFunc (rootNode->rightNode,applyFunc,nargs,args);
 }
 
-void avl_treeApplyFuncFixedArgs (AvlTree this1,void (*applyFunc) (),
-                                 int nargs,va_list args) {
+void avl_treeApplyFuncFixedArgs (AvlTree this1,void *applyFunc,int nargs,
+                                 va_list args) {
   /**
      Iterates through all the nodes and applies the function
      specified by the function pointer applyFunc() to the value stored
@@ -793,11 +809,10 @@ void avl_treeApplyFuncFixedArgs (AvlTree this1,void (*applyFunc) (),
      @param[in] nargs - number of arguments as integer
      @param[in] args - variable list of arguments
   */
-  avl_nodeApplyFunc (this1->rootTree->leftNode,(void (*)())applyFunc,
-                     nargs,args);
+  avl_nodeApplyFunc (this1->rootTree->leftNode,applyFunc,nargs,args);
 }
 
-void avl_treeApplyFunc (AvlTree this1,void (*applyFunc)(),int nargs,...) {
+void avl_treeApplyFunc (AvlTree this1,void *applyFunc,int nargs,...) {
   /**
      Iterates through all the nodes and applies the function
      specified by the function pointer applyFunc() to the value stored
@@ -814,8 +829,7 @@ void avl_treeApplyFunc (AvlTree this1,void (*applyFunc)(),int nargs,...) {
   va_list args;
 
   va_start (args,nargs);
-  avl_nodeApplyFunc (this1->rootTree->leftNode,(void (*)())applyFunc,
-                     nargs,args);
+  avl_nodeApplyFunc (this1->rootTree->leftNode,applyFunc,nargs,args);
   va_end (args);
   return;
 }
